﻿#include <iostream>
#include <vector>
using namespace std;

class MaxHeap {
private:
    vector<long long> heap;

    void heapifyUp(int index) {
        while (index > 0) {
            int parent = (index - 1) / 2;
            if (heap[index] > heap[parent]) {
                swap(heap[index], heap[parent]);
                index = parent;
            }
            else {
                break;
            }
        }
    }

    void heapifyDown(int index) {
        int leftChild, rightChild, largestChild;
        while (true) {
            leftChild = 2 * index + 1;
            rightChild = 2 * index + 2;
            largestChild = index;

            if (leftChild < heap.size() && heap[leftChild] > heap[largestChild]) {
                largestChild = leftChild;
            }

            if (rightChild < heap.size() && heap[rightChild] > heap[largestChild]) {
                largestChild = rightChild;
            }

            if (largestChild != index) {
                swap(heap[index], heap[largestChild]);
                index = largestChild;
            }
            else {
                break;
            }
        }
    }

public:
    void push(long long value) {
        heap.push_back(value);
        heapifyUp(heap.size() - 1);
    }

    long long pop() {
        if (heap.empty()) {
            cerr << "Heap is empty" << endl;
            return -1;
        }

        long long maxValue = heap[0];
        heap[0] = heap.back();
        heap.pop_back();
        heapifyDown(0);

        return maxValue;
    }

    bool empty() const {
        return heap.empty();
    }

    int size() const {
        return heap.size();
    }

    long long top() const {
        return heap[0];
    }
};

int main() {
    long long n, k;
    cin >> n >> k;

    MaxHeap ss;

    ss.push(n);
    for (int i = 0; i < k; i++) {
        long long a = ss.pop();

        long long x, y;
        if (a % 2 == 0) {
            x = a / 2;
            y = a / 2;
        }
        else {
            x = (a - 1) / 2 + 1;
            y = (a - 1) / 2;
        }

        ss.push(x);
        ss.push(y);
    }

    vector<long long> newss;
    while (!ss.empty()) {
        newss.push_back(ss.pop());
    }

    long long q;
    cin >> q;

    for (int i = 0; i < q; i++) {
        int r;
        cin >> r;
        cout << newss[r - 1] << ' ';
    }

    return 0;
}
